@@ -35,6 +35,17 @@
  * the GnomeVFS library. Common protocols are 'file', 'http', 'ftp', or 'smb'.
  * </para>
  * <para>
+ * In case the element-gnomevfssrc::iradio-mode property is set and the
+ * location is a http resource, gnomevfssrc will send special icecast http
+ * headers to the server to request additional icecast metainformation. If
+ * the server is not an icecast server, it will display the same behaviour
+ * as if the element-gnomevfssrc::iradio-mode property was not set. However,
+ * if the server is in fact an icecast server, gnomevfssrc will output
+ * data with a media type of application/x-icy, in which case you will
+ * need to use the #ICYDemux element as follow-up element to extract
+ * the icecast meta data and to determine the underlying media type.
+ * </para>
+ * <para>
  * Example pipeline:
  * <programlisting>
  * gst-launch -v gnomevfssrc location=file:///home/joe/foo.xyz ! fakesink
@@ -107,7 +118,6 @@ static GStaticMutex count_lock = G_STATIC_MUTEX_INIT;
 static gint ref_count = 0;
 static gboolean vfs_owner = FALSE;
 
-
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -329,7 +339,7 @@ gst_gnome_vfs_src_finalize (GObject * object)
  * URI interface support.
  */
 
-static guint
+static GstURIType
 gst_gnome_vfs_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -629,11 +639,11 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
 
   src = GST_GNOME_VFS_SRC (basesrc);
 
-  GST_DEBUG ("now at %llu, reading %lld, size %u", src->curoffset, offset,
+  GST_DEBUG ("now at %llu, reading from %lld, size %u", src->curoffset, offset,
       size);
 
   /* seek if required */
-  if (src->curoffset != offset) {
+  if (G_UNLIKELY (src->curoffset != offset)) {
     GST_DEBUG ("need to seek");
     if (src->seekable) {
       GST_DEBUG ("seeking to %lld", offset);
@@ -656,12 +666,13 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
 
   res = gnome_vfs_read (src->handle, data, size, &readbytes);
 
-  if (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK && readbytes == 0))
+  if (G_UNLIKELY (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK
+              && readbytes == 0)))
     goto eos;
 
   GST_BUFFER_SIZE (buf) = readbytes;
 
-  if (res != GNOME_VFS_OK)
+  if (G_UNLIKELY (res != GNOME_VFS_OK))
     goto read_failed;
 
   src->curoffset += readbytes;
@@ -682,7 +693,7 @@ cannot_seek:
   {
     GST_ELEMENT_ERROR (src, RESOURCE, SEEK, (NULL),
         ("Requested seek from %" G_GINT64_FORMAT " to %" G_GINT64_FORMAT
-            "on non-seekable stream", src->curoffset, offset));
+            " on non-seekable stream", src->curoffset, offset));
     return GST_FLOW_ERROR;
   }
 read_failed:
@@ -759,15 +770,40 @@ static gboolean
 gst_gnome_vfs_src_get_size (GstBaseSrc * basesrc, guint64 * size)
 {
   GstGnomeVFSSrc *src;
+  GnomeVFSFileInfo *info;
+  GnomeVFSFileInfoOptions options;
+  GnomeVFSResult res;
 
   src = GST_GNOME_VFS_SRC (basesrc);
 
-  GST_DEBUG_OBJECT (src, "size %" G_GUINT64_FORMAT, src->size);
+  *size = -1;
+  info = gnome_vfs_file_info_new ();
+  options = GNOME_VFS_FILE_INFO_DEFAULT | GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
+  res = gnome_vfs_get_file_info_from_handle (src->handle, info, options);
+  if (res == GNOME_VFS_OK) {
+    if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) != 0) {
+      *size = info->size;
+      GST_DEBUG_OBJECT (src, "from handle: %" G_GUINT64_FORMAT " bytes", *size);
+    } else if (src->own_handle && gnome_vfs_uri_is_local (src->uri)) {
+      GST_DEBUG_OBJECT (src,
+          "file size not known, file local, trying fallback");
+      res = gnome_vfs_get_file_info_uri (src->uri, info, options);
+      if (res == GNOME_VFS_OK &&
+          (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) != 0) {
+        *size = info->size;
+        GST_DEBUG_OBJECT (src, "from uri: %" G_GUINT64_FORMAT " bytes", *size);
+      }
+    }
+  } else {
+    GST_WARNING_OBJECT (src, "getting info failed: %s",
+        gnome_vfs_result_to_string (res));
+  }
+  gnome_vfs_file_info_unref (info);
 
-  if (src->size == (GnomeVFSFileSize) - 1)
+  if (*size == (GnomeVFSFileSize) - 1)
     return FALSE;
 
-  *size = src->size;
+  GST_DEBUG_OBJECT (src, "return size %" G_GUINT64_FORMAT, *size);
 
   return TRUE;
 }
@@ -777,7 +813,6 @@ static gboolean
 gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
 {
   GnomeVFSResult res;
-  GnomeVFSFileInfo *info;
   GstGnomeVFSSrc *src;
 
   src = GST_GNOME_VFS_SRC (basesrc);
@@ -785,9 +820,11 @@ gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
   gst_gnome_vfs_src_push_callbacks (src);
 
   if (src->uri != NULL) {
+    GnomeVFSOpenMode mode = GNOME_VFS_OPEN_READ;
+
     /* this can block... */
-    if ((res = gnome_vfs_open_uri (&src->handle, src->uri,
-                GNOME_VFS_OPEN_READ)) != GNOME_VFS_OK)
+    res = gnome_vfs_open_uri (&src->handle, src->uri, mode);
+    if (res != GNOME_VFS_OK)
       goto open_failed;
     src->own_handle = TRUE;
   } else if (!src->handle) {
@@ -796,23 +833,7 @@ gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
     src->own_handle = FALSE;
   }
 
-  src->size = (GnomeVFSFileSize) - 1;
-  info = gnome_vfs_file_info_new ();
-  if ((res = gnome_vfs_get_file_info_from_handle (src->handle,
-              info, GNOME_VFS_FILE_INFO_DEFAULT)) == GNOME_VFS_OK) {
-    if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) {
-      src->size = info->size;
-      GST_DEBUG_OBJECT (src, "size: %llu bytes", src->size);
-    } else
-      GST_LOG_OBJECT (src, "filesize not known");
-  } else {
-    GST_WARNING_OBJECT (src, "getting info failed: %s",
-        gnome_vfs_result_to_string (res));
-  }
-  gnome_vfs_file_info_unref (info);
-
-  if (gnome_vfs_seek (src->handle, GNOME_VFS_SEEK_CURRENT, 0)
-      == GNOME_VFS_OK) {
+  if (gnome_vfs_seek (src->handle, GNOME_VFS_SEEK_CURRENT, 0) == GNOME_VFS_OK) {
     src->seekable = TRUE;
   } else {
     src->seekable = FALSE;
@@ -829,14 +850,15 @@ open_failed:
     gst_gnome_vfs_src_pop_callbacks (src);
 
     if (res == GNOME_VFS_ERROR_NOT_FOUND ||
+        res == GNOME_VFS_ERROR_HOST_NOT_FOUND ||
         res == GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE) {
       GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
-          ("Could not open vfs file \"%s\" for reading: %s",
-              filename, gnome_vfs_result_to_string (res)));
+          ("Could not open vfs file \"%s\" for reading: %s (%d)",
+              filename, gnome_vfs_result_to_string (res), res));
     } else {
       GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-          ("Could not open vfs file \"%s\" for reading: %s",
-              filename, gnome_vfs_result_to_string (res)));
+          ("Could not open vfs file \"%s\" for reading: %s (%d)",
+              filename, gnome_vfs_result_to_string (res), res));
     }
     g_free (filename);
     return FALSE;
@@ -858,10 +880,15 @@ gst_gnome_vfs_src_stop (GstBaseSrc * basesrc)
   gst_gnome_vfs_src_pop_callbacks (src);
 
   if (src->own_handle) {
-    gnome_vfs_close (src->handle);
+    GnomeVFSResult res;
+
+    res = gnome_vfs_close (src->handle);
+    if (res != GNOME_VFS_OK) {
+      GST_ELEMENT_ERROR (src, RESOURCE, CLOSE, (NULL),
+          ("Could not close vfs handle: %s", gnome_vfs_result_to_string (res)));
+    }
     src->handle = NULL;
   }
-  src->size = (GnomeVFSFileSize) - 1;
   src->curoffset = 0;
 
   if (src->icy_caps) {
