@@ -224,8 +224,21 @@ _gst_audio_structure_set_list (GstStructure * structure,
  * @flag: a set of #GstAudioFieldFlag
  *
  * Do not use anymore.
- * @Deprecated: use gst_structure_set()
+ *
+ * Deprecated: use gst_structure_set()
  */
+#ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
+typedef enum
+{
+  GST_AUDIO_FIELD_RATE = (1 << 0),
+  GST_AUDIO_FIELD_CHANNELS = (1 << 1),
+  GST_AUDIO_FIELD_ENDIANNESS = (1 << 2),
+  GST_AUDIO_FIELD_WIDTH = (1 << 3),
+  GST_AUDIO_FIELD_DEPTH = (1 << 4),
+  GST_AUDIO_FIELD_SIGNED = (1 << 5),
+} GstAudioFieldFlag;
+#endif /* GST_DISABLE_DEPRECATED */
 void
 gst_audio_structure_set_int (GstStructure * structure, GstAudioFieldFlag flag)
 {
@@ -251,6 +264,7 @@ gst_audio_structure_set_int (GstStructure * structure, GstAudioFieldFlag flag)
     _gst_audio_structure_set_list (structure, "signed", G_TYPE_BOOLEAN, 2, TRUE,
         FALSE, NULL);
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 /**
  * gst_audio_buffer_clip:
@@ -261,7 +275,8 @@ gst_audio_structure_set_int (GstStructure * structure, GstAudioFieldFlag flag)
  *
  * Clip the the buffer to the given %GstSegment.
  *
- * After calling this function do not reference @buffer anymore.
+ * After calling this function the caller does not own a reference to 
+ * @buffer anymore.
  *
  * Returns: %NULL if the buffer is completely outside the configured segment,
  * otherwise the clipped buffer is returned.
@@ -290,7 +305,7 @@ gst_audio_buffer_clip (GstBuffer * buffer, GstSegment * segment, gint rate,
 
   if (!GST_BUFFER_TIMESTAMP_IS_VALID (buffer))
     /* No timestamp - assume the buffer is completely in the segment */
-    return buffer; 
+    return buffer;
 
   /* Get copies of the buffer metadata to change later. 
    * Calculate the missing values for the calculations,
