@@ -20,6 +20,53 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstaudiosink
+ * @short_description: Simple base class for audio sinks
+ * @see_also: #GstBaseAudioSink, #GstRingBuffer, #GstAudioSink.
+ *
+ * This is the most simple base class for audio sinks that only requires
+ * subclasses to implement a set of simple functions:
+ *
+ * <variablelist>
+ *   <varlistentry>
+ *     <term>open()</term>
+ *     <listitem><para>Open the device.</para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>prepare()</term>
+ *     <listitem><para>Configure the device with the specified format.</para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>write()</term>
+ *     <listitem><para>Write samples to the device.</para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>reset()</term>
+ *     <listitem><para>Unblock writes and flush the device.</para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>delay()</term>
+ *     <listitem><para>Get the number of samples written but not yet played 
+ *     by the device.</para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>unprepare()</term>
+ *     <listitem><para>Undo operations done by prepare.</para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>close()</term>
+ *     <listitem><para>Close the device.</para></listitem>
+ *   </varlistentry>
+ * </variablelist>
+ *
+ * All scheduling of samples and timestamps is done in this base class
+ * together with #GstBaseAudioSink using a default implementation of a
+ * #GstRingBuffer that uses threads.
+ *
+ * Last reviewed on 2006-09-27 (0.10.12)
+ */
+
 #include <string.h>
 
 #include "gstaudiosink.h"
@@ -35,9 +82,11 @@ GST_DEBUG_CATEGORY_STATIC (gst_audio_sink_debug);
         (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_AUDIORING_BUFFER,GstAudioRingBufferClass))
 #define GST_AUDIORING_BUFFER_GET_CLASS(obj) \
         (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_AUDIORING_BUFFER, GstAudioRingBufferClass))
+#define GST_AUDIORING_BUFFER_CAST(obj)        \
+        ((GstAudioRingBuffer *)obj)
 #define GST_IS_AUDIORING_BUFFER(obj)     \
         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_AUDIORING_BUFFER))
-#define GST_IS_AUDIORING_BUFFER_CLASS(obj)\
+#define GST_IS_AUDIORING_BUFFER_CLASS(klass)\
         (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_AUDIORING_BUFFER))
 
 typedef struct _GstAudioRingBuffer GstAudioRingBuffer;
@@ -119,7 +168,7 @@ gst_audioringbuffer_class_init (GstAudioRingBufferClass * klass)
   gstobject_class = (GstObjectClass *) klass;
   gstringbuffer_class = (GstRingBufferClass *) klass;
 
-  ring_parent_class = g_type_class_ref (GST_TYPE_RING_BUFFER);
+  ring_parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_audioringbuffer_dispose);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_audioringbuffer_finalize);
@@ -152,13 +201,13 @@ audioringbuffer_thread_func (GstRingBuffer * buf)
 {
   GstAudioSink *sink;
   GstAudioSinkClass *csink;
-  GstAudioRingBuffer *abuf = GST_AUDIORING_BUFFER (buf);
+  GstAudioRingBuffer *abuf = GST_AUDIORING_BUFFER_CAST (buf);
   WriteFunc writefunc;
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
 
-  GST_DEBUG ("enter thread");
+  GST_DEBUG_OBJECT (sink, "enter thread");
 
   writefunc = csink->write;
   if (writefunc == NULL)
@@ -175,11 +224,12 @@ audioringbuffer_thread_func (GstRingBuffer * buf)
       left = len;
       do {
         written = writefunc (sink, readptr + written, left);
-        GST_LOG ("transfered %d bytes of %d from segment %d", written, left,
-            readseg);
+        GST_LOG_OBJECT (sink, "transfered %d bytes of %d from segment %d",
+            written, left, readseg);
         if (written < 0 || written > left) {
-          GST_WARNING ("error writing data (reason: %s), skipping segment\n",
-              strerror (errno));
+          GST_WARNING_OBJECT (sink,
+              "error writing data (reason: %s), skipping segment",
+              g_strerror (errno));
           break;
         }
         left -= written;
@@ -194,18 +244,18 @@ audioringbuffer_thread_func (GstRingBuffer * buf)
       GST_OBJECT_LOCK (abuf);
       if (!abuf->running)
         goto stop_running;
-      GST_DEBUG ("signal wait");
+      GST_DEBUG_OBJECT (sink, "signal wait");
       GST_AUDIORING_BUFFER_SIGNAL (buf);
-      GST_DEBUG ("wait for action");
+      GST_DEBUG_OBJECT (sink, "wait for action");
       GST_AUDIORING_BUFFER_WAIT (buf);
-      GST_DEBUG ("got signal");
+      GST_DEBUG_OBJECT (sink, "got signal");
       if (!abuf->running)
         goto stop_running;
-      GST_DEBUG ("continue running");
+      GST_DEBUG_OBJECT (sink, "continue running");
       GST_OBJECT_UNLOCK (abuf);
     }
   }
-  GST_DEBUG ("exit thread");
+  GST_DEBUG_OBJECT (sink, "exit thread");
 
   return;
 
@@ -242,6 +292,10 @@ gst_audioringbuffer_dispose (GObject * object)
 static void
 gst_audioringbuffer_finalize (GObject * object)
 {
+  GstAudioRingBuffer *ringbuffer = GST_AUDIORING_BUFFER_CAST (object);
+
+  g_cond_free (ringbuffer->cond);
+
   G_OBJECT_CLASS (ring_parent_class)->finalize (object);
 }
 
@@ -318,7 +372,7 @@ gst_audioringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   buf->data = gst_buffer_new_and_alloc (spec->segtotal * spec->segsize);
   memset (GST_BUFFER_DATA (buf->data), 0, GST_BUFFER_SIZE (buf->data));
 
-  abuf = GST_AUDIORING_BUFFER (buf);
+  abuf = GST_AUDIORING_BUFFER_CAST (buf);
   abuf->running = TRUE;
 
   sink->thread =
@@ -346,7 +400,7 @@ gst_audioringbuffer_release (GstRingBuffer * buf)
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
-  abuf = GST_AUDIORING_BUFFER (buf);
+  abuf = GST_AUDIORING_BUFFER_CAST (buf);
 
   abuf->running = FALSE;
   GST_DEBUG ("signal wait");
@@ -418,7 +472,7 @@ gst_audioringbuffer_stop (GstRingBuffer * buf)
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
-  abuf = GST_AUDIORING_BUFFER (buf);
+  abuf = GST_AUDIORING_BUFFER_CAST (buf);
 
   /* unblock any pending writes to the audio device */
   if (csink->reset) {
