@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2007> Wim Taymans <wim.taymans@gmail.com>
+ * Copyright (C) <2007> Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,6 +19,7 @@
 
 /**
  * SECTION:element-uridecodebin
+ * @short_description: decoder for an uri
  *
  * Decodes data from a URI into raw media.
  */
@@ -32,9 +33,7 @@
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
 
-#include "gstfactorylists.h"
 #include "gstplay-marshal.h"
-#include "gstplay-enum.h"
 
 #define GST_TYPE_URI_DECODE_BIN \
   (gst_uri_decode_bin_get_type())
@@ -46,41 +45,23 @@
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_URI_DECODE_BIN))
 #define GST_IS_URI_DECODE_BIN_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_URI_DECODE_BIN))
-#define GST_URI_DECODE_BIN_CAST(obj) ((GstURIDecodeBin *) (obj))
 
 typedef struct _GstURIDecodeBin GstURIDecodeBin;
 typedef struct _GstURIDecodeBinClass GstURIDecodeBinClass;
 
-#define GST_URI_DECODE_BIN_GET_LOCK(dec) (((GstURIDecodeBin*)(dec))->lock)
-#define GST_URI_DECODE_BIN_LOCK(dec) (g_mutex_lock(GST_URI_DECODE_BIN_GET_LOCK(dec)))
-#define GST_URI_DECODE_BIN_UNLOCK(dec) (g_mutex_unlock(GST_URI_DECODE_BIN_GET_LOCK(dec)))
-
-/**
- * GstURIDecodeBin
- *
- * uridecodebin element struct
- */
 struct _GstURIDecodeBin
 {
   GstBin parent_instance;
 
-  GMutex *lock;                 /* lock for constructing */
-
-  GValueArray *factories;       /* factories we can use for selecting elements */
-
   gchar *uri;
   guint connection_speed;
   GstCaps *caps;
   gchar *encoding;
 
   gboolean is_stream;
-  guint64 buffer_duration;      /* When streaming, buffer duration (ns) */
-  guint buffer_size;            /* When streaming, buffer size (bytes) */
-
   GstElement *source;
-  GstElement *typefind;
-  guint have_type_id;           /* have-type signal id from typefind */
-  GSList *decodebins;
+  GstElement *queue;
+  GSList *decoders;
   GSList *srcpads;
   gint numpads;
 
@@ -88,8 +69,6 @@ struct _GstURIDecodeBin
   guint src_np_sig_id;          /* new-pad signal id */
   guint src_nmp_sig_id;         /* no-more-pads signal id */
   gint pending;
-
-  gboolean async_pending;       /* async-start has been emited */
 };
 
 struct _GstURIDecodeBinClass
@@ -106,11 +85,8 @@ struct _GstURIDecodeBinClass
   GValueArray *(*autoplug_factories) (GstElement * element, GstPad * pad,
       GstCaps * caps);
   /* signal fired to select from the proposed list of factories */
-    GstAutoplugSelectResult (*autoplug_select) (GstElement * element,
-      GstPad * pad, GstCaps * caps, GValueArray * factories);
-
-  /* emited when all data is decoded */
-  void (*drained) (GstElement * element);
+    gint (*autoplug_select) (GstElement * element, GstPad * pad, GstCaps * caps,
+      GValueArray * factories);
 };
 
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src%d",
@@ -125,7 +101,7 @@ static const GstElementDetails gst_uri_decode_bin_details =
 GST_ELEMENT_DETAILS ("URI Decoder",
     "Generic/Bin/Decoder",
     "Autoplug and decode an URI to raw media",
-    "Wim Taymans <wim.taymans@gmail.com>");
+    "Wim Taymans <wim@fluendo.com>");
 
 /* signals */
 enum
@@ -134,29 +110,22 @@ enum
   SIGNAL_AUTOPLUG_CONTINUE,
   SIGNAL_AUTOPLUG_FACTORIES,
   SIGNAL_AUTOPLUG_SELECT,
-  SIGNAL_DRAINED,
   LAST_SIGNAL
 };
 
 /* properties */
 #define DEFAULT_PROP_URI	    NULL
-#define DEFAULT_PROP_SOURCE	    NULL
 #define DEFAULT_CONNECTION_SPEED    0
 #define DEFAULT_CAPS                NULL
 #define DEFAULT_SUBTITLE_ENCODING   NULL
-#define DEFAULT_BUFFER_DURATION     -1
-#define DEFAULT_BUFFER_SIZE         -1
 
 enum
 {
   PROP_0,
   PROP_URI,
-  PROP_SOURCE,
   PROP_CONNECTION_SPEED,
   PROP_CAPS,
   PROP_SUBTITLE_ENCODING,
-  PROP_BUFFER_SIZE,
-  PROP_BUFFER_DURATION,
   PROP_LAST
 };
 
@@ -170,8 +139,6 @@ static void gst_uri_decode_bin_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_uri_decode_bin_finalize (GObject * obj);
 
-static void handle_message (GstBin * bin, GstMessage * msg);
-
 static gboolean gst_uri_decode_bin_query (GstElement * element,
     GstQuery * query);
 static GstStateChangeReturn gst_uri_decode_bin_change_state (GstElement *
@@ -201,32 +168,6 @@ _gst_boolean_accumulator (GSignalInvocationHint * ihint,
   return myboolean;
 }
 
-static gboolean
-_gst_array_accumulator (GSignalInvocationHint * ihint,
-    GValue * return_accu, const GValue * handler_return, gpointer dummy)
-{
-  gpointer array;
-
-  array = g_value_get_boxed (handler_return);
-  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
-    g_value_set_boxed (return_accu, array);
-
-  return FALSE;
-}
-
-static gboolean
-_gst_select_accumulator (GSignalInvocationHint * ihint,
-    GValue * return_accu, const GValue * handler_return, gpointer dummy)
-{
-  GstAutoplugSelectResult res;
-
-  res = g_value_get_enum (handler_return);
-  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
-    g_value_set_enum (return_accu, res);
-
-  return FALSE;
-}
-
 static gboolean
 gst_uri_decode_bin_autoplug_continue (GstElement * element, GstPad * pad,
     GstCaps * caps)
@@ -235,25 +176,6 @@ gst_uri_decode_bin_autoplug_continue (GstElement * element, GstPad * pad,
   return TRUE;
 }
 
-static GValueArray *
-gst_uri_decode_bin_autoplug_factories (GstElement * element, GstPad * pad,
-    GstCaps * caps)
-{
-  GValueArray *result;
-
-  GST_DEBUG_OBJECT (element, "finding factories");
-
-  /* return all compatible factories for caps */
-  result =
-      gst_factory_list_filter (GST_URI_DECODE_BIN_CAST (element)->factories,
-      caps);
-
-  GST_DEBUG_OBJECT (element, "autoplug-factories returns %p", result);
-
-  return result;
-}
-
-
 static void
 gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
 {
@@ -271,46 +193,28 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_URI,
       g_param_spec_string ("uri", "URI", "URI to decode",
-          DEFAULT_PROP_URI, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  g_object_class_install_property (gobject_class, PROP_SOURCE,
-      g_param_spec_object ("source", "Source", "Source object used",
-          GST_TYPE_ELEMENT, G_PARAM_READABLE));
+          DEFAULT_PROP_URI, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_CONNECTION_SPEED,
       g_param_spec_uint ("connection-speed", "Connection Speed",
           "Network connection speed in kbps (0 = unknown)",
-          0, G_MAXUINT, DEFAULT_CONNECTION_SPEED,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, G_MAXUINT, DEFAULT_CONNECTION_SPEED, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_CAPS,
       g_param_spec_boxed ("caps", "Caps",
           "The caps on which to stop decoding. (NULL = default)",
-          GST_TYPE_CAPS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_CAPS, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_SUBTITLE_ENCODING,
       g_param_spec_string ("subtitle-encoding", "subtitle encoding",
           "Encoding to assume if input subtitles are not in UTF-8 encoding. "
           "If not set, the GST_SUBTITLE_ENCODING environment variable will "
           "be checked for an encoding to use. If that is not set either, "
-          "ISO-8859-15 will be assumed.", NULL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  g_object_class_install_property (gobject_class, PROP_BUFFER_SIZE,
-      g_param_spec_int ("buffer-size", "Buffer size (bytes)",
-          "Buffer size when buffering network streams",
-          -1, G_MAXINT, DEFAULT_BUFFER_SIZE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_BUFFER_DURATION,
-      g_param_spec_int64 ("buffer-duration", "Buffer duration (ns)",
-          "Buffer duration when buffering network streams",
-          -1, G_MAXINT64, DEFAULT_BUFFER_DURATION,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "ISO-8859-15 will be assumed.", NULL, G_PARAM_READWRITE));
 
   /**
    * GstURIDecodeBin::unknown-type:
-   * @pad: the new pad containing caps that cannot be resolved to a 'final'
-   * stream type.
+   * @pad: the new pad containing caps that cannot be resolved to a 'final' stream type.
    * @caps: the #GstCaps of the pad that cannot be resolved.
    *
    * This signal is emitted when a pad for which there is no further possible
@@ -319,7 +223,7 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
   gst_uri_decode_bin_signals[SIGNAL_UNKNOWN_TYPE] =
       g_signal_new ("unknown-type", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstURIDecodeBinClass, unknown_type),
-      NULL, NULL, gst_marshal_VOID__OBJECT_BOXED, G_TYPE_NONE, 2,
+      NULL, NULL, gst_marshal_VOID__OBJECT_OBJECT, G_TYPE_NONE, 2,
       GST_TYPE_PAD, GST_TYPE_CAPS);
 
   /**
@@ -339,7 +243,7 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
       g_signal_new ("autoplug-continue", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstURIDecodeBinClass,
           autoplug_continue), _gst_boolean_accumulator, NULL,
-      gst_play_marshal_BOOLEAN__OBJECT_BOXED, G_TYPE_BOOLEAN, 2, GST_TYPE_PAD,
+      gst_play_marshal_BOOLEAN__OBJECT_OBJECT, G_TYPE_BOOLEAN, 2, GST_TYPE_PAD,
       GST_TYPE_CAPS);
 
   /**
@@ -357,8 +261,8 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
   gst_uri_decode_bin_signals[SIGNAL_AUTOPLUG_FACTORIES] =
       g_signal_new ("autoplug-factories", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstURIDecodeBinClass,
-          autoplug_factories), _gst_array_accumulator, NULL,
-      gst_play_marshal_BOXED__OBJECT_BOXED, G_TYPE_VALUE_ARRAY, 2,
+          autoplug_factories), NULL, NULL,
+      gst_play_marshal_BOXED__OBJECT_OBJECT, G_TYPE_VALUE_ARRAY, 2,
       GST_TYPE_PAD, GST_TYPE_CAPS);
 
   /**
@@ -379,49 +283,25 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
   gst_uri_decode_bin_signals[SIGNAL_AUTOPLUG_SELECT] =
       g_signal_new ("autoplug-select", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstURIDecodeBinClass,
-          autoplug_select), _gst_select_accumulator, NULL,
-      gst_play_marshal_ENUM__OBJECT_BOXED_OBJECT,
-      GST_TYPE_AUTOPLUG_SELECT_RESULT, 3, GST_TYPE_PAD, GST_TYPE_CAPS,
-      GST_TYPE_ELEMENT_FACTORY);
-
-  /**
-   * GstURIDecodeBin::drained:
-   *
-   * This signal is emitted when the data for the current uri is played.
-   */
-  gst_uri_decode_bin_signals[SIGNAL_DRAINED] =
-      g_signal_new ("drained", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstURIDecodeBinClass, drained), NULL, NULL,
-      gst_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
+          autoplug_select), NULL, NULL,
+      gst_play_marshal_INT__OBJECT_OBJECT_BOXED, G_TYPE_INT, 3, GST_TYPE_PAD,
+      GST_TYPE_CAPS, G_TYPE_VALUE_ARRAY);
 
   gstelement_class->query = GST_DEBUG_FUNCPTR (gst_uri_decode_bin_query);
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_uri_decode_bin_change_state);
 
-  gstbin_class->handle_message = GST_DEBUG_FUNCPTR (handle_message);
-
   klass->autoplug_continue =
       GST_DEBUG_FUNCPTR (gst_uri_decode_bin_autoplug_continue);
-  klass->autoplug_factories =
-      GST_DEBUG_FUNCPTR (gst_uri_decode_bin_autoplug_factories);
 }
 
 static void
 gst_uri_decode_bin_init (GstURIDecodeBin * dec, GstURIDecodeBinClass * klass)
 {
-  /* first filter out the interesting element factories */
-  dec->factories = gst_factory_list_get_elements (GST_FACTORY_LIST_DECODER);
-
-  dec->lock = g_mutex_new ();
-
   dec->uri = g_strdup (DEFAULT_PROP_URI);
   dec->connection_speed = DEFAULT_CONNECTION_SPEED;
   dec->caps = DEFAULT_CAPS;
   dec->encoding = g_strdup (DEFAULT_SUBTITLE_ENCODING);
-
-  dec->buffer_duration = DEFAULT_BUFFER_DURATION;
-  dec->buffer_size = DEFAULT_BUFFER_SIZE;
 }
 
 static void
@@ -429,39 +309,12 @@ gst_uri_decode_bin_finalize (GObject * obj)
 {
   GstURIDecodeBin *dec = GST_URI_DECODE_BIN (obj);
 
-  g_mutex_free (dec->lock);
   g_free (dec->uri);
   g_free (dec->encoding);
-  if (dec->factories) {
-    g_value_array_free (dec->factories);
-    dec->factories = NULL;
-  }
 
   G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
-static void
-gst_uri_decode_bin_set_encoding (GstURIDecodeBin * dec, const gchar * encoding)
-{
-  GSList *walk;
-
-  GST_URI_DECODE_BIN_LOCK (dec);
-
-  /* set property first */
-  GST_OBJECT_LOCK (dec);
-  g_free (dec->encoding);
-  dec->encoding = g_strdup (encoding);
-  GST_OBJECT_UNLOCK (dec);
-
-  /* set the property on all decodebins now */
-  for (walk = dec->decodebins; walk; walk = g_slist_next (walk)) {
-    GObject *decodebin = G_OBJECT (walk->data);
-
-    g_object_set (decodebin, "subtitle-encoding", encoding, NULL);
-  }
-  GST_URI_DECODE_BIN_UNLOCK (dec);
-}
-
 static void
 gst_uri_decode_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -488,13 +341,10 @@ gst_uri_decode_bin_set_property (GObject * object, guint prop_id,
       GST_OBJECT_UNLOCK (dec);
       break;
     case PROP_SUBTITLE_ENCODING:
-      gst_uri_decode_bin_set_encoding (dec, g_value_get_string (value));
-      break;
-    case PROP_BUFFER_SIZE:
-      dec->buffer_size = g_value_get_int (value);
-      break;
-    case PROP_BUFFER_DURATION:
-      dec->buffer_duration = g_value_get_int64 (value);
+      GST_OBJECT_LOCK (dec);
+      g_free (dec->encoding);
+      dec->encoding = g_value_dup_string (value);
+      GST_OBJECT_UNLOCK (dec);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -514,11 +364,6 @@ gst_uri_decode_bin_get_property (GObject * object, guint prop_id,
       g_value_set_string (value, dec->uri);
       GST_OBJECT_UNLOCK (dec);
       break;
-    case PROP_SOURCE:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_object (value, dec->source);
-      GST_OBJECT_UNLOCK (dec);
-      break;
     case PROP_CONNECTION_SPEED:
       GST_OBJECT_LOCK (dec);
       g_value_set_uint (value, dec->connection_speed / 1000);
@@ -534,52 +379,18 @@ gst_uri_decode_bin_get_property (GObject * object, guint prop_id,
       g_value_set_string (value, dec->encoding);
       GST_OBJECT_UNLOCK (dec);
       break;
-    case PROP_BUFFER_SIZE:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_int (value, dec->buffer_size);
-      GST_OBJECT_UNLOCK (dec);
-      break;
-    case PROP_BUFFER_DURATION:
-      GST_OBJECT_LOCK (dec);
-      g_value_set_int64 (value, dec->buffer_duration);
-      GST_OBJECT_UNLOCK (dec);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-static void
-do_async_start (GstURIDecodeBin * dbin)
-{
-  GstMessage *message;
-
-  dbin->async_pending = TRUE;
-
-  message = gst_message_new_async_start (GST_OBJECT_CAST (dbin), FALSE);
-  parent_class->handle_message (GST_BIN_CAST (dbin), message);
-}
-
-static void
-do_async_done (GstURIDecodeBin * dbin)
-{
-  GstMessage *message;
-
-  if (dbin->async_pending) {
-    message = gst_message_new_async_done (GST_OBJECT_CAST (dbin));
-    parent_class->handle_message (GST_BIN_CAST (dbin), message);
-
-    dbin->async_pending = FALSE;
-  }
-}
-
 #define DEFAULT_QUEUE_SIZE          (3 * GST_SECOND)
 #define DEFAULT_QUEUE_MIN_THRESHOLD ((DEFAULT_QUEUE_SIZE * 30) / 100)
 #define DEFAULT_QUEUE_THRESHOLD     ((DEFAULT_QUEUE_SIZE * 95) / 100)
 
 static void
-unknown_type_cb (GstElement * element, GstPad * pad, GstCaps * caps,
+unknown_type (GstElement * element, GstPad * pad, GstCaps * caps,
     GstURIDecodeBin * decoder)
 {
   gchar *capsstr;
@@ -613,7 +424,7 @@ no_more_pads_full (GstElement * element, gboolean subs,
   /* setup phase */
   GST_DEBUG_OBJECT (element, "no more pads, %d pending", decoder->pending);
 
-  GST_URI_DECODE_BIN_LOCK (decoder);
+  GST_OBJECT_LOCK (decoder);
   final = (decoder->pending == 0);
 
   /* nothing pending, we can exit */
@@ -629,7 +440,7 @@ no_more_pads_full (GstElement * element, gboolean subs,
   final = (decoder->pending == 0);
 
 done:
-  GST_URI_DECODE_BIN_UNLOCK (decoder);
+  GST_OBJECT_UNLOCK (decoder);
 
   if (final)
     gst_element_no_more_pads (GST_ELEMENT_CAST (decoder));
@@ -661,7 +472,7 @@ source_no_more_pads (GstElement * element, GstURIDecodeBin * bin)
  * found a new raw pad.  
  */
 static void
-new_decoded_pad_cb (GstElement * element, GstPad * pad, gboolean last,
+new_decoded_pad (GstElement * element, GstPad * pad, gboolean last,
     GstURIDecodeBin * decoder)
 {
   GstPad *newpad;
@@ -670,53 +481,18 @@ new_decoded_pad_cb (GstElement * element, GstPad * pad, gboolean last,
   GST_DEBUG_OBJECT (element, "new decoded pad, name: <%s>. Last: %d",
       GST_PAD_NAME (pad), last);
 
-  GST_URI_DECODE_BIN_LOCK (decoder);
+  GST_OBJECT_LOCK (decoder);
   padname = g_strdup_printf ("src%d", decoder->numpads);
   decoder->numpads++;
-  GST_URI_DECODE_BIN_UNLOCK (decoder);
+  GST_OBJECT_UNLOCK (decoder);
 
   newpad = gst_ghost_pad_new (padname, pad);
   g_free (padname);
 
-  /* store ref to the ghostpad so we can remove it */
-  g_object_set_data (G_OBJECT (pad), "uridecodebin.ghostpad", newpad);
-
   gst_pad_set_active (newpad, TRUE);
   gst_element_add_pad (GST_ELEMENT_CAST (decoder), newpad);
 }
 
-static void
-pad_removed_cb (GstElement * element, GstPad * pad, GstURIDecodeBin * decoder)
-{
-  GstPad *ghost;
-
-  GST_DEBUG_OBJECT (element, "pad removed name: <%s:%s>",
-      GST_DEBUG_PAD_NAME (pad));
-
-  /* we only care about srcpads */
-  if (!GST_PAD_IS_SRC (pad))
-    return;
-
-  if (!(ghost = g_object_get_data (G_OBJECT (pad), "uridecodebin.ghostpad")))
-    goto no_ghost;
-
-  /* unghost the pad */
-  gst_ghost_pad_set_target (GST_GHOST_PAD_CAST (ghost), NULL);
-
-  /* deactivate and remove */
-  gst_pad_set_active (pad, FALSE);
-  gst_element_remove_pad (GST_ELEMENT_CAST (decoder), ghost);
-
-  return;
-
-  /* ERRORS */
-no_ghost:
-  {
-    GST_WARNING_OBJECT (element, "no ghost pad found");
-    return;
-  }
-}
-
 /* helper function to lookup stuff in lists */
 static gboolean
 array_has_value (const gchar * values[], const gchar * value)
@@ -796,13 +572,7 @@ gen_source_element (GstURIDecodeBin * decoder)
     g_object_set (source, "connection-speed",
         decoder->connection_speed / 1000, NULL);
   }
-  if (g_object_class_find_property (G_OBJECT_GET_CLASS (source),
-          "subtitle-encoding")) {
-    GST_DEBUG_OBJECT (decoder,
-        "setting subtitle-encoding=%s to source element", decoder->encoding);
-    g_object_set (G_OBJECT (source), "subtitle-encoding", decoder->encoding,
-        NULL);
-  }
+
   return source;
 
   /* ERRORS */
@@ -948,7 +718,7 @@ analyse_source (GstURIDecodeBin * decoder, gboolean * is_raw,
 
         /* caps on source pad are all raw, we can add the pad */
         if (*is_raw)
-          new_decoded_pad_cb (decoder->source, pad, FALSE, decoder);
+          new_decoded_pad (decoder->source, pad, FALSE, decoder);
         break;
     }
   }
@@ -984,15 +754,15 @@ remove_decoders (GstURIDecodeBin * bin)
 {
   GSList *walk;
 
-  for (walk = bin->decodebins; walk; walk = g_slist_next (walk)) {
+  for (walk = bin->decoders; walk; walk = g_slist_next (walk)) {
     GstElement *decoder = GST_ELEMENT_CAST (walk->data);
 
     GST_DEBUG_OBJECT (bin, "removing old decoder element");
     gst_element_set_state (decoder, GST_STATE_NULL);
     gst_bin_remove (GST_BIN_CAST (bin), decoder);
   }
-  g_slist_free (bin->decodebins);
-  bin->decodebins = NULL;
+  g_slist_free (bin->decoders);
+  bin->decoders = NULL;
 }
 
 static void
@@ -1015,8 +785,6 @@ static void
 proxy_unknown_type_signal (GstElement * element, GstPad * pad, GstCaps * caps,
     GstURIDecodeBin * dec)
 {
-  GST_DEBUG_OBJECT (dec, "unknown-type signaled");
-
   g_signal_emit (G_OBJECT (dec),
       gst_uri_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, pad, caps);
 }
@@ -1030,9 +798,6 @@ proxy_autoplug_continue_signal (GstElement * element, GstPad * pad,
   g_signal_emit (G_OBJECT (dec),
       gst_uri_decode_bin_signals[SIGNAL_AUTOPLUG_CONTINUE], 0, pad, caps,
       &result);
-
-  GST_DEBUG_OBJECT (dec, "autoplug-continue returned %d", result);
-
   return result;
 }
 
@@ -1045,41 +810,25 @@ proxy_autoplug_factories_signal (GstElement * element, GstPad * pad,
   g_signal_emit (G_OBJECT (dec),
       gst_uri_decode_bin_signals[SIGNAL_AUTOPLUG_FACTORIES], 0, pad, caps,
       &result);
-
-  GST_DEBUG_OBJECT (dec, "autoplug-factories returned %p", result);
-
   return result;
 }
 
-static GstAutoplugSelectResult
+static gint
 proxy_autoplug_select_signal (GstElement * element, GstPad * pad,
-    GstCaps * caps, GstElementFactory * factory, GstURIDecodeBin * dec)
+    GstCaps * caps, GValueArray * array, GstURIDecodeBin * dec)
 {
-  GstAutoplugSelectResult result;
+  gint result;
 
   g_signal_emit (G_OBJECT (dec),
-      gst_uri_decode_bin_signals[SIGNAL_AUTOPLUG_SELECT], 0, pad, caps, factory,
+      gst_uri_decode_bin_signals[SIGNAL_AUTOPLUG_SELECT], 0, pad, caps, array,
       &result);
-
-  GST_DEBUG_OBJECT (dec, "autoplug-select returned %d", result);
-
   return result;
 }
 
-static void
-proxy_drained_signal (GstElement * element, GstURIDecodeBin * dec)
-{
-  GST_DEBUG_OBJECT (dec, "drained signaled");
-
-  g_signal_emit (G_OBJECT (dec),
-      gst_uri_decode_bin_signals[SIGNAL_DRAINED], 0, NULL);
-}
-
-/* make a decodebin and connect to all the signals */
 static GstElement *
-make_decoder (GstURIDecodeBin * decoder)
+make_decoder (GstURIDecodeBin * decoder, gboolean use_queue)
 {
-  GstElement *decodebin;
+  GstElement *result, *decodebin;
 
   /* now create the decoder element */
   decodebin = gst_element_factory_make ("decodebin2", NULL);
@@ -1095,29 +844,51 @@ make_decoder (GstURIDecodeBin * decoder)
       G_CALLBACK (proxy_autoplug_factories_signal), decoder);
   g_signal_connect (G_OBJECT (decodebin), "autoplug-select",
       G_CALLBACK (proxy_autoplug_select_signal), decoder);
-  g_signal_connect (G_OBJECT (decodebin), "drained",
-      G_CALLBACK (proxy_drained_signal), decoder);
+
+  if (use_queue) {
+    GstElement *queue;
+    GstPad *gpad, *pad;
+
+    queue = gst_element_factory_make ("queue2", NULL);
+    if (!queue)
+      goto no_queue2;
+
+    /* configure the queue as a buffering element */
+    g_object_set (G_OBJECT (queue), "use-buffering", TRUE, NULL);
+
+    result = gst_bin_new ("source-bin");
+
+    gst_bin_add (GST_BIN_CAST (result), queue);
+    gst_bin_add (GST_BIN_CAST (result), decodebin);
+
+    gst_element_link (queue, decodebin);
+
+    pad = gst_element_get_pad (queue, "sink");
+    gpad = gst_ghost_pad_new (GST_PAD_NAME (pad), pad);
+    gst_object_unref (pad);
+
+    gst_pad_set_active (gpad, TRUE);
+    gst_element_add_pad (GST_ELEMENT_CAST (result), gpad);
+  } else {
+    result = decodebin;
+  }
 
   /* set up callbacks to create the links between decoded data
    * and video/audio/subtitle rendering/output. */
   g_signal_connect (G_OBJECT (decodebin),
-      "new-decoded-pad", G_CALLBACK (new_decoded_pad_cb), decoder);
-  g_signal_connect (G_OBJECT (decodebin),
-      "pad-removed", G_CALLBACK (pad_removed_cb), decoder);
+      "new-decoded-pad", G_CALLBACK (new_decoded_pad), decoder);
   g_signal_connect (G_OBJECT (decodebin), "no-more-pads",
       G_CALLBACK (no_more_pads), decoder);
   g_signal_connect (G_OBJECT (decodebin),
-      "unknown-type", G_CALLBACK (unknown_type_cb), decoder);
+      "unknown-type", G_CALLBACK (unknown_type), decoder);
   g_object_set_data (G_OBJECT (decodebin), "pending", "1");
-  g_object_set (G_OBJECT (decodebin), "subtitle-encoding", decoder->encoding,
-      NULL);
   decoder->pending++;
 
-  gst_bin_add (GST_BIN_CAST (decoder), decodebin);
+  gst_bin_add (GST_BIN_CAST (decoder), result);
 
-  decoder->decodebins = g_slist_prepend (decoder->decodebins, decodebin);
+  decoder->decoders = g_slist_prepend (decoder->decoders, result);
 
-  return decodebin;
+  return result;
 
   /* ERRORS */
 no_decodebin:
@@ -1126,117 +897,11 @@ no_decodebin:
         (_("Could not create \"decodebin2\" element.")), (NULL));
     return NULL;
   }
-}
-
-static void
-type_found (GstElement * typefind, guint probability,
-    GstCaps * caps, GstURIDecodeBin * decoder)
-{
-  GstElement *dec_elem, *queue;
-
-  GST_DEBUG_OBJECT (decoder, "typefind found caps %" GST_PTR_FORMAT, caps);
-
-  dec_elem = make_decoder (decoder);
-  if (!dec_elem)
-    goto no_decodebin;
-
-  queue = gst_element_factory_make ("queue2", NULL);
-  if (!queue)
-    goto no_queue2;
-
-  g_object_set (G_OBJECT (queue), "use-buffering", TRUE, NULL);
-  /* g_object_set (G_OBJECT (queue), "temp-location", "temp", NULL); */
-
-  /* Disable max-size-buffers */
-  g_object_set (G_OBJECT (queue), "max-size-buffers", 0, NULL);
-
-  /* If buffer size or duration are set, set them on the queue2 element */
-  if (decoder->buffer_size != -1)
-    g_object_set (G_OBJECT (queue), "max-size-bytes",
-        decoder->buffer_size, NULL);
-  if (decoder->buffer_duration != -1)
-    g_object_set (G_OBJECT (queue), "max-size-time",
-        decoder->buffer_duration, NULL);
-
-  gst_bin_add (GST_BIN_CAST (decoder), queue);
-
-  if (!gst_element_link (typefind, queue))
-    goto could_not_link;
-
-  g_object_set (G_OBJECT (dec_elem), "sink-caps", caps, NULL);
-
-  if (!gst_element_link (queue, dec_elem))
-    goto could_not_link;
-
-  gst_element_set_state (dec_elem, GST_STATE_PLAYING);
-  gst_element_set_state (queue, GST_STATE_PLAYING);
-
-  do_async_done (decoder);
-
-  return;
-
-  /* ERRORS */
-no_decodebin:
-  {
-    /* error was posted */
-    return;
-  }
-could_not_link:
-  {
-    GST_ELEMENT_ERROR (decoder, CORE, NEGOTIATION,
-        (NULL), ("Can't link typefind to decodebin2 element"));
-    return;
-  }
 no_queue2:
   {
     GST_ELEMENT_ERROR (decoder, CORE, MISSING_PLUGIN,
         (_("Could not create \"queue2\" element.")), (NULL));
-    return;
-  }
-}
-
-/* setup a streaming source. This will first plug a typefind element to the
- * source. After we find the type, we decide to plug a queue2 and continue to
- * plug a decodebin2 starting from the found caps */
-static gboolean
-setup_streaming (GstURIDecodeBin * decoder)
-{
-  GstElement *typefind;
-
-  /* now create the decoder element */
-  typefind = gst_element_factory_make ("typefind", NULL);
-  if (!typefind)
-    goto no_typefind;
-
-  gst_bin_add (GST_BIN_CAST (decoder), typefind);
-
-  if (!gst_element_link (decoder->source, typefind))
-    goto could_not_link;
-
-  decoder->typefind = typefind;
-
-  /* connect a signal to find out when the typefind element found
-   * a type */
-  decoder->have_type_id =
-      g_signal_connect (G_OBJECT (decoder->typefind), "have-type",
-      G_CALLBACK (type_found), decoder);
-
-  do_async_start (decoder);
-
-  return TRUE;
-
-  /* ERRORS */
-no_typefind:
-  {
-    GST_ELEMENT_ERROR (decoder, CORE, MISSING_PLUGIN,
-        (_("Could not create \"typefind\" element.")), (NULL));
-    return FALSE;
-  }
-could_not_link:
-  {
-    GST_ELEMENT_ERROR (decoder, CORE, NEGOTIATION,
-        (NULL), ("Can't link source to typefind element"));
-    return FALSE;
+    return NULL;
   }
 }
 
@@ -1269,20 +934,18 @@ source_new_pad (GstElement * element, GstPad * pad, GstURIDecodeBin * bin)
   GstElement *decoder;
   gboolean is_raw;
 
-  GST_URI_DECODE_BIN_LOCK (bin);
   GST_DEBUG_OBJECT (bin, "Found new pad %s.%s in source element %s",
       GST_DEBUG_PAD_NAME (pad), GST_ELEMENT_NAME (element));
 
   /* if this is a pad with all raw caps, we can expose it */
   if (has_all_raw_caps (pad, &is_raw) && is_raw) {
     /* it's all raw, create output pads. */
-    GST_URI_DECODE_BIN_UNLOCK (bin);
-    new_decoded_pad_cb (element, pad, FALSE, bin);
+    new_decoded_pad (element, pad, FALSE, bin);
     return;
   }
 
   /* not raw, create decoder */
-  decoder = make_decoder (bin);
+  decoder = make_decoder (bin, FALSE);
   if (!decoder)
     goto no_decodebin;
 
@@ -1293,7 +956,6 @@ source_new_pad (GstElement * element, GstPad * pad, GstURIDecodeBin * bin)
   GST_DEBUG_OBJECT (bin, "linked decoder to new pad");
 
   gst_element_set_state (decoder, GST_STATE_PLAYING);
-  GST_URI_DECODE_BIN_UNLOCK (bin);
 
   return;
 
@@ -1301,14 +963,12 @@ source_new_pad (GstElement * element, GstPad * pad, GstURIDecodeBin * bin)
 no_decodebin:
   {
     /* error was posted */
-    GST_URI_DECODE_BIN_UNLOCK (bin);
     return;
   }
 could_not_link:
   {
     GST_ELEMENT_ERROR (bin, CORE, NEGOTIATION,
         (NULL), ("Can't link source to decoder element"));
-    GST_URI_DECODE_BIN_UNLOCK (bin);
     return;
   }
 }
@@ -1334,9 +994,6 @@ setup_source (GstURIDecodeBin * decoder)
    * handled by the application right after. */
   gst_bin_add (GST_BIN_CAST (decoder), decoder->source);
 
-  /* notify of the new source used */
-  g_object_notify (G_OBJECT (decoder), "source");
-
   /* remove the old decoders now, if any */
   remove_decoders (decoder);
 
@@ -1373,24 +1030,17 @@ setup_source (GstURIDecodeBin * decoder)
     g_object_set_data (G_OBJECT (decoder->source), "pending", "1");
     decoder->pending++;
   } else {
-    if (decoder->is_stream) {
-      GST_DEBUG_OBJECT (decoder, "Setting up streaming");
-      /* do the stream things here */
-      if (!setup_streaming (decoder))
-        goto streaming_failed;
-    } else {
-      GstElement *dec_elem;
+    GstElement *dec_elem;
 
-      /* no streaming source, we can link now */
-      GST_DEBUG_OBJECT (decoder, "Plugging decodebin to source");
+    GST_DEBUG_OBJECT (decoder, "Pluggin decodebin to source");
 
-      dec_elem = make_decoder (decoder);
-      if (!dec_elem)
-        goto no_decoder;
+    /* no dynamic source, we can link now */
+    dec_elem = make_decoder (decoder, decoder->is_stream);
+    if (!dec_elem)
+      goto no_decoder;
 
-      if (!gst_element_link (decoder->source, dec_elem))
-        goto could_not_link;
-    }
+    if (!gst_element_link (decoder->source, dec_elem))
+      goto could_not_link;
   }
   return TRUE;
 
@@ -1411,11 +1061,6 @@ no_decoder:
     /* message was posted */
     return FALSE;
   }
-streaming_failed:
-  {
-    /* message was posted */
-    return FALSE;
-  }
 could_not_link:
   {
     GST_ELEMENT_ERROR (decoder, CORE, NEGOTIATION,
@@ -1424,107 +1069,6 @@ could_not_link:
   }
 }
 
-static void
-value_list_append_structure_list (GValue * list_val, GstStructure ** first,
-    GList * structure_list)
-{
-  GList *l;
-
-  for (l = structure_list; l != NULL; l = l->next) {
-    GValue val = { 0, };
-
-    if (*first == NULL)
-      *first = gst_structure_copy ((GstStructure *) l->data);
-
-    g_value_init (&val, GST_TYPE_STRUCTURE);
-    g_value_take_boxed (&val, gst_structure_copy ((GstStructure *) l->data));
-    gst_value_list_append_value (list_val, &val);
-    g_value_unset (&val);
-  }
-}
-
-/* if it's a redirect message with multiple redirect locations we might
- * want to pick a different 'best' location depending on the required
- * bitrates and the connection speed */
-static GstMessage *
-handle_redirect_message (GstURIDecodeBin * dec, GstMessage * msg)
-{
-  const GValue *locations_list, *location_val;
-  GstMessage *new_msg;
-  GstStructure *new_structure = NULL;
-  GList *l_good = NULL, *l_neutral = NULL, *l_bad = NULL;
-  GValue new_list = { 0, };
-  guint size, i;
-
-  GST_DEBUG_OBJECT (dec, "redirect message: %" GST_PTR_FORMAT, msg);
-  GST_DEBUG_OBJECT (dec, "connection speed: %u", dec->connection_speed);
-
-  if (dec->connection_speed == 0 || msg->structure == NULL)
-    return msg;
-
-  locations_list = gst_structure_get_value (msg->structure, "locations");
-  if (locations_list == NULL)
-    return msg;
-
-  size = gst_value_list_get_size (locations_list);
-  if (size < 2)
-    return msg;
-
-  /* maintain existing order as much as possible, just sort references
-   * with too high a bitrate to the end (the assumption being that if
-   * bitrates are given they are given for all interesting streams and
-   * that the you-need-at-least-version-xyz redirect has the same bitrate
-   * as the lowest referenced redirect alternative) */
-  for (i = 0; i < size; ++i) {
-    const GstStructure *s;
-    gint bitrate = 0;
-
-    location_val = gst_value_list_get_value (locations_list, i);
-    s = (const GstStructure *) g_value_get_boxed (location_val);
-    if (!gst_structure_get_int (s, "minimum-bitrate", &bitrate) || bitrate <= 0) {
-      GST_DEBUG_OBJECT (dec, "no bitrate: %" GST_PTR_FORMAT, s);
-      l_neutral = g_list_append (l_neutral, (gpointer) s);
-    } else if (bitrate > dec->connection_speed) {
-      GST_DEBUG_OBJECT (dec, "bitrate too high: %" GST_PTR_FORMAT, s);
-      l_bad = g_list_append (l_bad, (gpointer) s);
-    } else if (bitrate <= dec->connection_speed) {
-      GST_DEBUG_OBJECT (dec, "bitrate OK: %" GST_PTR_FORMAT, s);
-      l_good = g_list_append (l_good, (gpointer) s);
-    }
-  }
-
-  g_value_init (&new_list, GST_TYPE_LIST);
-  value_list_append_structure_list (&new_list, &new_structure, l_good);
-  value_list_append_structure_list (&new_list, &new_structure, l_neutral);
-  value_list_append_structure_list (&new_list, &new_structure, l_bad);
-  gst_structure_set_value (new_structure, "locations", &new_list);
-  g_value_unset (&new_list);
-
-  g_list_free (l_good);
-  g_list_free (l_neutral);
-  g_list_free (l_bad);
-
-  new_msg = gst_message_new_element (msg->src, new_structure);
-  gst_message_unref (msg);
-
-  GST_DEBUG_OBJECT (dec, "new redirect message: %" GST_PTR_FORMAT, new_msg);
-  return new_msg;
-}
-
-static void
-handle_message (GstBin * bin, GstMessage * msg)
-{
-  if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ELEMENT && msg->structure != NULL
-      && gst_structure_has_name (msg->structure, "redirect")) {
-    /* sort redirect messages based on the connection speed. This simplifies
-     * the user of this element as it can in most cases just pick the first item
-     * of the sorted list as a good redirection candidate. It can of course
-     * choose something else from the list if it has a better way. */
-    msg = handle_redirect_message (GST_URI_DECODE_BIN (bin), msg);
-  }
-  GST_BIN_CLASS (parent_class)->handle_message (bin, msg);
-}
-
 /* generic struct passed to all query fold methods
  * FIXME, move to core.
  */
@@ -1821,7 +1365,6 @@ gst_uri_decode_bin_change_state (GstElement * element,
       remove_decoders (decoder);
       remove_pads (decoder);
       remove_source (decoder);
-      do_async_done (decoder);
       break;
     default:
       break;
@@ -1840,10 +1383,8 @@ setup_failed:
   }
 }
 
-gboolean gst_decode_bin_plugin_init (GstPlugin * plugin);
-
 static gboolean
-gst_uri_decode_bin_plugin_init (GstPlugin * plugin)
+plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_uri_decode_bin_debug, "uridecodebin", 0,
       "URI decoder element");
@@ -1852,24 +1393,12 @@ gst_uri_decode_bin_plugin_init (GstPlugin * plugin)
   GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
       LOCALEDIR);
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif /* ENABLE_NLS */
 
   return gst_element_register (plugin, "uridecodebin", GST_RANK_NONE,
       GST_TYPE_URI_DECODE_BIN);
 }
 
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  if (!gst_decode_bin_plugin_init (plugin))
-    return FALSE;
-  if (!gst_uri_decode_bin_plugin_init (plugin))
-    return FALSE;
-
-  return TRUE;
-}
-
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "uridecodebin",
